<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'tiny_equation', language 'en'.
 *
 * @package    tiny_equation
 * @copyright  2022 Huong Nguyen <huongnv13@gmail.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['buttontitle'] = 'Equation editor';
$string['cursorinfo'] = 'An arrow indicates the position that new elements from the element library will be inserted.';
$string['editequation'] = 'Edit equation using <a href="{$a}" target="_blank">TeX</a>';
$string['equation:use'] = 'Use TinyMCE equation editor';
$string['librarygroup1'] = 'Operators';
$string['librarygroup1_desc'] = 'TeX commands listed on the operators tab.';
$string['librarygroup2'] = 'Arrows';
$string['librarygroup2_desc'] = 'TeX commands listed on the arrows tab.';
$string['librarygroup3'] = 'Greek symbols';
$string['librarygroup3_desc'] = 'TeX commands listed on the Greek symbols tab.';
$string['librarygroup4'] = 'Advanced';
$string['librarygroup4_desc'] = 'TeX commands listed on the advanced tab.';
$string['modaltitle'] = 'Equation editor';
$string['pluginname'] = 'Equation editor';
$string['preview'] = 'Equation preview';
$string['privacy:metadata'] = 'The equation editor for TinyMCE does not store any personal data.';
$string['saveequation'] = 'Save equation';
$string['settings'] = 'Equation editor settings';

// Deprecated since Moodle 4.5.
$string['helplinktext'] = 'Equation helper';
