H5PEditor.language.core = {
  missingTranslation: '[Saknar översättning :key]',
  loading: 'Laddar, vänta ...',
  selectLibrary: 'Välj det bibliotek som du vill använda för ditt innehåll.',
  unknownFieldPath: 'Kunde inte hitta ":path".',
  notImageField: '":path" är inte en bild.',
  notImageOrDimensionsField: '":path" är inte en bild eller dimensionsfält.',
  requiredProperty: ':property krävs och måste ha ett värde.',
  onlyNumbers: 'Värdet :property kan endast innehålla nummer.',
  illegalDecimalNumber: ':property kan endast innehålla nummer med max :decimals decimaler.',
  exceedsMax: 'Värdet :property överskrider maxvärdet :max.',
  listExceedsMax: 'Listan överskrider maxvärdet :max poster.',
  belowMin: 'Värdet :property är under minimum :min.',
  listBelowMin: 'Listan behöver mninst :min poster för att innehållet ska fungera korrekt.',
  outOfStep: 'Värdet :property kan endast ändras i steg om :step.',
  add: 'Lägg till',
  addFile: 'Lägg till fil',
  removeFile: 'Ta bort fil',
  confirmRemoval: 'Är du säker på att du vill  ta bort :type?',
  removeImage: 'Ta bort bild',
  confirmImageRemoval: 'Detta kommer ta bort din bild. Är du säker?',
  changeFile: 'Byt ut fil',
  changeLibrary: 'Ändra innehållstyp?',
  semanticsError: 'Semantikfel: :error',
  missingProperty: 'Fält :index saknar dess :property egenskap.',
  expandCollapse: 'Expandera/Komprimera',
  addEntity: 'Lägg till :entity',
  tooLong: 'Fältvärdet är för långt, får max innehålla :max tecken eller färre.',
  invalidFormat: 'Fältvärdet innehåller ett ogiltigt format eller tecken som är förbjudna.',
  confirmChangeLibrary:
    'Om du gör detta så kommer du förlora allt du gjort med nuvarande innehållstyp. Är du säker på att du vill byta innehållstyp?',
  commonFields: 'Textersättningar och översättningar',
  commonFieldsDescription: 'Här kan du ändra eller översätta texter som används i detta innehåll.',
  uploading: 'Laddar upp, vänta ...',
  noFollow: 'Kan inte följa fält ":path".',
  editCopyright: 'Redigera copyright',
  close: 'Stäng',
  tutorial: 'Guide',
  editMode: 'Redigeringsläge',
  listLabel: 'Lista',
  uploadError: 'Fel vid filuppladdning',
  fileToLarge: 'Filen som du försöker ladda upp kan vara för stor.',
  unknownFileUploadError: 'Okänt fel vid filuppladdning',
  noSemantics: 'Fel, kunde inte ladda formatet för innehållstypen',
  editImage: 'Redigera bild',
  saveLabel: 'Spara',
  cancelLabel: 'Avbryt',
  resetToOriginalLabel: 'Återställ till original',
  loadingImageEditor: 'Laddar bildredigerare,vänta ...',
  selectFiletoUpload: 'Välj fil att ladda upp',
  or: 'eller',
  enterAudioUrl: 'Lägg till URL för ljudkälla',
  enterVideoUrl: 'Enter video URL',
  enterAudioTitle: 'Klistra in länk eller annan URL till ljudkälla',
  enterVideoTitle: 'Klistra in YouTube-länk eller annan URL till videokälla',
  uploadAudioTitle: 'Ladda upp ljudfil',
  uploadVideoTitle: 'Ladda upp videofil',
  addVideoDescription:
    'H5P supports all external video sources formatted as mp4, webm or ogv, like Vimeo Pro, and has support for YouTube and Panopto links.',
  insert: 'Infoga',
  cancel: 'Avbryt',
  height: 'höjd',
  width: 'bredd',
  textField: 'textfält',
  numberField: 'nummerfält',
  orderItemUp: 'Flytta post uppåt',
  orderItemDown: 'Flytta post nedåt',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Ta bort post',
  hubPanelLabel: 'Välj innehållstyp',
  importantInstructions: 'Viktiga instruktioner',
  showImportantInstructions: 'Visa instruktioner',
  hideImportantInstructions: 'Dölj viktiga instruktioner',
  hide: 'Dölj',
  example: 'Exempel',
  createContentTabLabel: 'Skapa innehåll',
  uploadTabLabel: 'Ladda upp',
  uploadPlaceholder: 'Ingen fil vald',
  uploadInstructionsTitle: 'Ladda upp en H5P-fil.',
  uploadInstructionsContent:
    'Du kan börja med exempel från <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Ladda upp en fil',
  uploadFileButtonChangeLabel: 'Byt ut fil',
  uploadingThrobber: 'Laddar upp ...',
  uploadSuccess: ':title uppladdad!',
  unableToInterpretError: 'Kunde inte tolka svar.',
  unableToInterpretSolution: 'Kontrollera din fellogg.',
  h5pFileWrongExtensionTitle: 'Den valda filen kunde inte laddas upp',
  h5pFileWrongExtensionContent: 'Endast filer med filändelsen .h5p tillåts.',
  h5pFileValidationFailedTitle: 'Kunde inte validera H5P-fil.',
  h5pFileValidationFailedContent:
    'Kontrollera att uppladdad H5P innehåller giltigt H5P innehåll. H5P filer som enbart innehåller bibliotek ska laddas upp via sidan H5P-bibliotek.',
  h5pFileUploadServerErrorTitle: 'H5P-filen kunde inte laddas upp',
  h5pFileUploadServerErrorContent: 'Ett fel har uppstått. Kontrollera din fellogg på servern för mer information.',
  contentTypeSectionAll: 'Alla innehållstyper',
  searchResults: 'Sökrsultat',
  contentTypeSearchFieldPlaceholder: 'Sök innehållstyper',
  contentTypeInstallButtonLabel: 'Installera',
  contentTypeInstallingButtonLabel: 'Installerar',
  contentTypeUseButtonLabel: 'Använd',
  contentTypeDetailButtonLabel: 'Uppgifter',
  contentTypeUpdateButtonLabel: 'Uppdatera',
  contentTypeUpdatingButtonLabel: 'Uppdaterar',
  contentTypeGetButtonLabel: 'Hämta',
  contentTypeBackButtonLabel: 'Tillbaka',
  contentTypeIconAltText: 'Ikon',
  contentTypeInstallSuccess: ':contentType installerad!',
  contentTypeUpdateSuccess: ':contentType uppdaterad!',
  contentTypeInstallError: ':contentType kunde inte installeras. Kontakta din administratör.',
  contentTypeLicensePanelTitle: 'Licens',
  contentTypeDemoButtonLabel: 'Innehållsdemo',
  numResults: ':num resultat',
  show: 'Visa',
  recentlyUsedFirst: 'Senast använda först',
  popularFirst: 'Populära först',
  newestFirst: 'Nyast först',
  aToZ: 'A till Ö',
  noResultsFound: 'Inga resultat',
  noResultsFoundDesc: 'Det finns ingen innehållstyp som matchar din sökning.',
  readMore: 'Läs mer',
  readLess: 'Läs mindre',
  contentTypeOwner: 'Av :owner',
  contentTypeUnsupportedApiVersionTitle: 'Denna innehållstyp kräver en nyare core-version',
  contentTypeUnsupportedApiVersionContent:
    'Kontakta din systemadministratör för att få tillgång till nödvändiga uppdateringar',
  contentTypeUpdateAvailable: 'Uppdatering tillgänglig',
  contentTypeRestricted: 'Begränsad innehållstyp',
  contentTypeRestrictedDesc: 'Användningen av denna innehållstyp har begränsats av en administratör.',
  contentTypeNotInstalled: 'Innehållstyp inte installerad',
  contentTypeNotInstalledDesc: 'Du har inte rättighet att installera innehållstyper.',
  theContentType: 'innehållstypen',
  currentMenuSelected: 'nuvarande urval',
  errorCommunicatingHubTitle: 'Kan inte kommunicera med hubben.',
  errorCommunicatingHubContent: 'Ett fel uppstod. Försök igen.',
  warningNoContentTypesInstalled: 'Du har inte någon innehållstyp installerad.',
  warningChangeBrowsingToSeeResults:
    'Klicka <em>Alla</em> för att få listan med alla innehållstyper som du kan installera.',
  warningUpdateAvailableTitle: 'En ny version av :contentType är tillgänglig.',
  warningUpdateAvailableBody: 'Uppdatera till senaste version för en förbättrad upplevelse.',
  licenseDescription:
    'Vissa egenskaper för denna licens framgår nedan. Klicka på info-ikonen ovanför för att läsa licenstexten i original.',
  licenseModalTitle: 'Licensuppgifter',
  licenseModalSubtitle: 'Välj en licens för att ta del av information om korrekt användning',
  licenseUnspecified: 'Ospecificerad',
  licenseCanUseCommercially: 'Kan använda kommersiellt',
  licenseCanModify: 'Kan modifiera',
  licenseCanDistribute: 'Kan distribuera',
  licenseCanSublicense: 'Kan underlicensiera',
  licenseCanHoldLiable: 'Kan hållas ansvarig',
  licenseCannotHoldLiable: 'Kan inte hållas ansvarig',
  licenseMustIncludeCopyright: 'Måste inkludera copyright',
  licenseMustIncludeLicense: 'Måste inkludera licens',
  licenseFetchDetailsFailed: 'Misslyckades med hämtning av licensuppgifter',
  imageLightboxTitle: 'Bilder',
  imageLightBoxProgress: ':num av :total',
  nextImage: 'Nästa bild',
  previousImage: 'Föregående bild',
  screenshots: 'Skärmbilder',
  reloadButtonLabel: 'Ladda om',
  videoQuality: 'Etikett för videokvalitet',
  videoQualityDescription:
    'Denna etikett underlättar för användaren att identifiera nuvarande kvalitet på videon. T ex 1080p, 720p, HD eller mobil',
  videoQualityDefaultLabel: 'Kvalitet :index',
  noContentTypesAvailable: 'Inga innehållstyper är tillgängliga',
  noContentTypesAvailableDesc:
    'Din webbplats har problem med att ansluta till H5P.org och listan över tillgängliga innehållstyper.',
  contentTypeCacheOutdated: 'Lista över innehållstyper behöver uppdateras.',
  contentTypeCacheOutdatedDesc:
    'Din webbplats har problem med att ansluta till H5P.org för att leta efter uppdateringar av innehållstyper. Det kan hända att du inte kan uppdatera eller installera nya innehållstyper.',
  tryAgain: 'Försök igen',
  getHelp: 'Få hjälp',
  untitled: 'Utan titel :libraryTitle',
  title: 'Titel',
  metadata: 'Metadata',
  addTitle: 'Lägg till titel',
  usedForSearchingReportsAndCopyrightInformation: 'Används för sökning, rapporter och copyright-information',
  metadataSharingAndLicensingInfo: 'Metadata (info om delning och licensiering)',
  fillInTheFieldsBelow: 'Fyll i fälten nedan',
  saveMetadata: 'Spara metadata',
  addAuthor: 'Spara författare',
  confirmRemoveAuthor: 'Är du säker på att du vill ta bort denna författare?',
  addNewChange: 'Lägg till ny ändring',
  confirmDeleteChangeLog: 'Är du säker på att du vill radera denna post i ändringsloggen?',
  changelogDescription:
    'Vissa licenser kräver att ändringar till originalet, eller derivat, loggas och visas. Du kan logga dina ändringar här av licensskäl eller bara för din egen eller andras skull.',
  logThisChange: 'Logga denna ändring',
  newChangeHasBeenLogged: 'Ny ändring har loggats',
  loggedChanges: 'Loggade ändringar',
  noChangesHaveBeenLogged: 'Inga ändringar har loggats',
  errorHeader: 'Ett fel uppstod',
  errorCalculatingMaxScore:
    'Kunde inte beräkna maxpoäng för detta innehåll. Maxpoäng förväntas vara 0. Kontakta din administratör om detta inte är korrekt.',
  maxScoreSemanticsMissing: 'Kunde inte hitta förväntad semantik i detta innehåll.',
  copyButton: 'Kopiera',
  copiedButton: 'Kopierad',
  pasteButton: 'Klistra in',
  pasteAndReplaceButton: 'Klistra in & ersätt',
  pasteContent: 'Ersätt innehåll',
  confirmPasteContent:
    'Om du gör detta så kommer du ersätta befintligt innehåll med ditt kopierade innehåll från urklipp. Nuvarande innehåll kommer att försvinna. Är du säker på att du vill fortsätta?',
  confirmPasteButtonText: 'Ersätt innehåll',
  copyToClipboard: 'Kopiera H5P-innehåll till urklipp',
  copiedToClipboard: 'Innehåll är kopierat till urklipp',
  pasteFromClipboard: 'Klistra in H5P-innehåll från urklipp',
  pasteAndReplaceFromClipboard: 'Ersätt nuvarande inehåll med H5P-innehåll från urklipp',
  pasteNoContent: 'Inget H5P-innehåll i urklipp',
  pasteError: 'Kan inte klistra in från urklipp',
  pasteContentNotSupported: 'Innehållet i urklipp stöds inte i detta sammanhang',
  pasteContentRestricted: 'Innehållet i urklipp har begränsats på denna webbplats',
  pasteTooOld:
    'Innehållet i urklipp har en lägre version (:clip) än vad som stöds i detta sammanhang (:local), om möjligt, uppgradera innehållet du vill klistra in och försök sen igen.',
  pasteTooNew:
    'Innehållet i urklipp har en högre version (:clip) än vad som stöds i detta sammanhang (:local), om möjligt, uppgradera innehållet här först och försök sen igen.',
  ok: 'OK',
  avTablistLabel: 'Infoga genom',
  tabTitleBasicFileUpload: 'Filuppladdning',
  tabTitleInputLinkURL: 'Länk/URL',
  errorTooHighVersion: 'Parametrar innehåller %used där endast %supported eller tidigare stöds.',
  errorNotSupported: 'Parametrar innehåller %used som inte stöds.',
  errorParamsBroken: 'Parametrar är trasiga .',
  libraryMissing: 'Saknar obligatoriskt bibliotek %lib.',
  scriptMissing: 'Kunde inte ladda upp uppgraderingsscript för %lib.',
  language: 'Språk',
  noLanguagesSupported: 'Inga språk stöd',
  changeLanguage: 'Ändra språk till :language?',
  thisWillPotentially:
    'Detta kan nollställa alla textändringar och översättningar här. Du kan inte ångra detta. Själva innehållet kommer däremot inte att påverkas. Vill du fortsätta?',
  notAllTextsChanged: 'Inte alla texter ändrades, det finns endast delvis stöd för :language.',
  contributeTranslations:
    'Om du vill bidra till översättningen av :language så kan du läsa mer här <a href=":url" target="_new">contributing translations to H5P</a>',
  unknownLibrary: "Tyvärr, innehållstypen '%lib' är inte installerad på denna webbplats.",
  proceedButtonLabel: 'Fortsätt för att spara',
  enterFullscreenButtonLabel: 'Helskärmsläge',
  exitFullscreenButtonLabel: 'Avsluta helskärm',
  reuseSuccess: ':title was successfully imported from the H5P Hub.',
  noContentHeader: 'No Fitting Content?',
  noContentSuggestion: 'Create one yourself!',
  tutorials: 'Tutorials',
  contentSectionAll: 'All shared content',
  popularContent: 'Popular Content',
  allPopular: 'All Popular',
  newOnTheHub: 'New on the Hub',
  allNew: 'All New',
  filterBy: 'Filter by',
  filter: 'Filter',
  filters: {
    level: {
      dropdownLabel: 'Level',
      dialogHeader: 'Select level of education',
      dialogButtonLabel: 'Filter level of education',
    },
    language: {
      dropdownLabel: 'Language',
      dialogHeader: 'Select language(s)',
      dialogButtonLabel: 'Filter languages',
      searchPlaceholder: 'Type to search for languages',
    },
    reviewed: {
      dropdownLabel: 'Reviewed',
      dialogHeader: 'Reviewed Content',
      dialogButtonLabel: 'Filter',
      optionLabel: 'Show only reviewed content',
    },
    contentTypes: {
      dropdownLabel: 'Content types',
      dialogHeader: 'Select Content type(s)',
      dialogButtonLabel: 'Filter Content Types',
      searchPlaceholder: 'Type to search for content types',
    },
    disciplines: {
      dropdownLabel: 'Discipline',
      dialogHeader: 'Select Your Discipline',
      dialogButtonLabel: 'Filter Disciplines',
      searchPlaceholder: 'Type to search for disciplines',
    },
    licenses: {
      dropdownLabel: 'License',
      dialogHeader: 'Select preferred rights of use',
      dialogButtonLabel: 'Filter licenses',
      options: { modified: 'Can be modified', commercial: 'Allows commercial use' },
    },
  },
  clearFilters: 'Clear all filters',
  contentSearchFieldPlaceholder: 'Search for Content',
  loadingContentTitle: 'We are loading content for you...',
  loadingContentSubtitle: 'Please wait',
  by: 'By',
  dropdownButton: 'Open dropdown',
  paginationNavigation: 'Pagination navigation',
  page: 'Page',
  currentPage: 'Current page',
  nextPage: 'Go to next page',
  previousPage: 'Go to previous page',
  contentPreviewButtonLabel: 'Preview',
  contentDownloadButtonLabel: 'Get Content',
  reuseContentTabLabel: 'Get Shared Content',
  contentPublisherPanelHeader: 'Publisher Info',
  noContentFoundDesc: 'There is no content that matches your search criteria.',
  h5pType: 'H5P Type',
  level: 'Level',
  size: 'Size',
  failedFetchingData: 'Failed fetching data',
  filterErrorMessage: 'Something went wrong. Please reload the page.',
  in: 'in',
  navigateToParent: 'Navigate to parent',
  a11yTitleShowLabel: 'Show label for AT',
  a11yTitleHideLabel: 'Hide label for AT',
};
