{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/insert_image_modal_details_footer

    Insert image details footer template.

    Example context (json):
    {
        "elementid": "exampleId"
    }

}}
<div class="row">
    <!-- First Column -->
    <div class="col-md-6 d-flex align-items-center p-0">
        <!-- Row 1: URL related label -->
        <span class="tiny_image_filename text-truncate me-1"></span>
    </div>
    <!-- Column 2: Saving, canceling, browsing repositories buttons -->
    <div class="col-md-6 text-end mt-2 mt-md-0 p-0">
        <!-- Row 1: Cancel button -->
        <button type="button" class="btn btn-secondary" data-action="cancel">{{#str}} cancel, moodle {{/str}}</button>
        <!-- Row 2: Save button -->
        <button class="tiny_image_urlentrysubmit btn btn-primary" type="submit">{{#str}} saveimage, tiny_media {{/str}}</button>
    </div>
</div>

