{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/embed/footer/insert_media_footer

    Insert media footer template.

    Example context (json):
    {
        "elementid": "id",
        "isInsertThumbnail": false,
        "showDropzone": true,
        "showFilePicker": true,
        "showImageFilePicker": true
    }

}}
<div class="row">
    <div class="col-md-6 d-flex align-items-center p-0">
        {{#showDropzone}}
            <label for="{{elementid}}_tiny_media_from_url_entry" class="me-1">{{#str}} enterurlor, tiny_media {{/str}}</label>
        {{/showDropzone}}
        {{^showDropzone}}
            <label for="{{elementid}}_tiny_media_from_url_entry" class="me-1">{{#str}} enterurl, tiny_media {{/str}}</label>
        {{/showDropzone}}
        <input
            name="urlentry"
            class="tiny_media_from_url_entry form-control w-50 me-1"
            type="url"
            id="{{elementid}}_tiny_media_from_url_entry"
            size="32"
            value="{{src}}"
        >
        <button disabled class="tiny_media_add_url btn btn-secondary me-1" type="button">{{#str}} addurl, tiny_media {{/str}}</button>
    </div>
    <div class="col-md-6 text-end mt-2 mt-md-0 p-0">
        <button type="button" class="btn btn-secondary" data-action="cancel">{{#str}} cancel, moodle {{/str}}</button>
        {{#showFilePicker}}
            <button class="openmediabrowser btn btn-secondary" type="button">{{#str}} browseembedimagerepositories, tiny_media {{/str}}</button>
        {{/showFilePicker}}
        {{#showImageFilePicker}}
            <button class="openimagebrowser btn btn-secondary" type="button">{{#str}} browseembedimagerepositories, tiny_media {{/str}}</button>
        {{/showImageFilePicker}}
    </div>
</div>
