<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'tiny_autosave', language 'en'.
 *
 * @package    tiny_autosave
 * @copyright  2022 Andrew Lyons <andrew@nicols.co.uk>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['autosave:use'] = 'Use TinyMCE autosave';
$string['pluginname'] = 'Autosave';
$string['privacy:metadata:database:tiny_autosave:userid'] = 'The user ID of the user who created the autosave session';
$string['privacy:metadata:database:tiny_autosave:drafttext'] = 'The text content of the autosave session';
$string['privacy:metadata:database:tiny_autosave:timemodified'] = 'The time that the autosave session was last modified';
$string['privacy:metadata:database:tiny_autosave'] = 'A table storing autosave session data for the TinyMCE editor';

// Deprecated since Moodle 4.5.
$string['helplinktext'] = 'Autosave';
