H5PEditor.language.core = {
  missingTranslation: '[Tradução em falta :key]',
  loading: 'A carregar. Por favor, aguarde...',
  selectLibrary: 'Selecione a biblioteca que pretende usar no seu conteúdo.',
  unknownFieldPath: 'Não foi possível encontrar ":path".',
  notImageField: '":path" não é uma imagem.',
  notImageOrDimensionsField: '":path" não é uma imagem ou campo de dimensão.',
  requiredProperty: ':property é obrigatória e deve ter um valor.',
  onlyNumbers: 'O valor de :property apenas pode conter números.',
  illegalDecimalNumber: ':property apenas pode conter números com o máximo de :decimals casas decimais.',
  exceedsMax: 'O valor de :property excede o máximo de :max.',
  listExceedsMax: 'A lista excede o número máxima de :max itens.',
  belowMin: 'O valor de :property é menor que o mínimo de :min.',
  listBelowMin: 'A lista precisa de pelo menos :min itens para que o conteúdo funcione corretamente.',
  outOfStep: 'O valor de :property apenas pode ser mudado numa variação de :step.',
  add: 'Adicionar',
  addFile: 'Adicionar ficheiro',
  removeFile: 'Remover ficheiro',
  confirmRemoval: 'Tem a certeza de que pretende remover :type?',
  removeImage: 'Remover imagem',
  confirmImageRemoval: 'Esta ação removerá a sua imagem. Tem a certeza de que pretende continuar?',
  changeFile: 'Alterar ficheiro',
  changeLibrary: 'Pretende alterar o tipo de conteúdo?',
  semanticsError: 'Erro de semântica: :error',
  missingProperty: 'O campo :index não contém a propriedade :property.',
  expandCollapse: 'Expandir/Contrair',
  addEntity: 'Adicionar :entity',
  tooLong: 'O valor do campo é muito longo. Deve conter :max caracteres no máximo.',
  invalidFormat: 'O valor do campo contém um formato inválido ou caracteres não permitidos.',
  confirmChangeLibrary:
    'Se continuar, perderá todo o trabalho concluído com o tipo atual de conteúdo. Tem a certeza de que pretende alterar o tipo de conteúdo?',
  commonFields: 'Substituição de texto e traduções',
  commonFieldsDescription: 'Aqui pode editar as configurações ou traduzir textos usados neste conteúdo.',
  uploading: 'A enviar. Por favor, aguarde...',
  noFollow: 'Não é possível seguir o campo ":path".',
  editCopyright: 'Editar direitos de autor',
  close: 'Fechar',
  tutorial: 'Tutorial',
  editMode: 'Modo de edição',
  listLabel: 'Lista',
  uploadError: 'Ocorreu um erro ao enviar o ficheiro',
  fileToLarge: 'O ficheiro que está a tentar enviar talvez seja muito grande.',
  unknownFileUploadError: 'Erro desconhecido no envio do ficheiro',
  noSemantics: 'Ocorreu um erro. Não foi possível carregar o formulário do tipo de conteúdo.',
  editImage: 'Editar imagem',
  saveLabel: 'Guardar',
  cancelLabel: 'Cancelar',
  resetToOriginalLabel: 'Restabelecer o original',
  loadingImageEditor: 'A carregar o editor de imagem. Por favor, aguarde...',
  selectFiletoUpload: 'Selecione o ficheiro para enviar',
  or: 'ou',
  enterAudioUrl: 'Insira o URL do áudio',
  enterVideoUrl: 'Enter video URL',
  enterAudioTitle: 'Cole aqui a hiperligação do áudio',
  enterVideoTitle: 'Cole aqui o URL do vídeo ou a hiperligação do YouTube',
  uploadAudioTitle: 'Enviar ficheiro de áudio',
  uploadVideoTitle: 'Enviar ficheiro de vídeo',
  addVideoDescription:
    'H5P supports all external video sources formatted as mp4, webm or ogv, like Vimeo Pro, and has support for YouTube and Panopto links.',
  insert: 'Inserir',
  cancel: 'Cancelar',
  height: 'altura',
  width: 'largura',
  textField: 'campo de texto',
  numberField: 'campo numérico',
  orderItemUp: 'Mover item para cima',
  orderItemDown: 'Mover item para baixo',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Remover item',
  hubPanelLabel: 'Selecionar tipo de conteúdo',
  importantInstructions: 'Instruções importantes',
  showImportantInstructions: 'Mostrar instruções',
  hideImportantInstructions: 'Ocultar instruções',
  hide: 'Ocultar',
  example: 'Exemplo',
  createContentTabLabel: 'Criar conteúdo',
  uploadTabLabel: 'Carregar',
  uploadPlaceholder: 'Nenhum ficheiro selecionado',
  uploadInstructionsTitle: 'Carregar um ficheiro H5P.',
  uploadInstructionsContent:
    'Pode começar com exemplos existentes em <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Carregar um ficheiro',
  uploadFileButtonChangeLabel: 'Alterar ficheiro',
  uploadingThrobber: 'A carregar...',
  uploadSuccess: ':title foi carregado com sucesso!',
  unableToInterpretError: 'Não foi possível interpretar a resposta.',
  unableToInterpretSolution: 'Por favor, verifique o registo de erros.',
  h5pFileWrongExtensionTitle: 'Não foi possível carregar o ficheiro',
  h5pFileWrongExtensionContent: 'Apenas são permitidos ficheiros com a extensão .h5p.',
  h5pFileValidationFailedTitle: 'Não foi possível validar o ficheiro H5P.',
  h5pFileValidationFailedContent:
    'Certifique-se que o ficheiros H5P carregado possui conteúdo H5P válido. Ficheiros H5P com bibliotecas devem ser enviados através da página de Bibliotecas H5P.',
  h5pFileUploadServerErrorTitle: 'Não foi possível enviar o ficheiro H5P',
  h5pFileUploadServerErrorContent:
    'Ocorreu um erro inesperado. Para mais informações, consulte o registo de erros do servidor.',
  contentTypeSectionAll: 'Todos os tipos de conteúdo',
  searchResults: 'Procurar resultados',
  contentTypeSearchFieldPlaceholder: 'Procurar por tipos de conteúdo',
  contentTypeInstallButtonLabel: 'Instalar',
  contentTypeInstallingButtonLabel: 'A instalar',
  contentTypeUseButtonLabel: 'Usar',
  contentTypeDetailButtonLabel: 'Detalhes',
  contentTypeUpdateButtonLabel: 'Atualizar',
  contentTypeUpdatingButtonLabel: 'A atualizar',
  contentTypeGetButtonLabel: 'Obter',
  contentTypeBackButtonLabel: 'Voltar',
  contentTypeIconAltText: 'Ícone',
  contentTypeInstallSuccess: ':contentType instalado com sucesso!',
  contentTypeUpdateSuccess: ':contentType atualizado com sucesso!',
  contentTypeInstallError: ':contentType não pôde ser instalado. Contacte o administrador.',
  contentTypeLicensePanelTitle: 'Licença',
  contentTypeDemoButtonLabel: 'Demonstração de conteúdo',
  numResults: ':num resultados',
  show: 'Mostrar',
  recentlyUsedFirst: 'Recentemente usados primeiro',
  popularFirst: 'Populares primeiro',
  newestFirst: 'Novos primeiro',
  aToZ: 'Ordem alfabética',
  noResultsFound: 'Nenhum resultado encontrado',
  noResultsFoundDesc: 'Não existe nenhum tipo de conteúdo que corresponda aos critérios da pesquisa.',
  readMore: 'Ver mais',
  readLess: 'Ver menos',
  contentTypeOwner: 'Por :owner',
  contentTypeUnsupportedApiVersionTitle: 'Este tipo de conteúdo requer uma nova versão do core',
  contentTypeUnsupportedApiVersionContent:
    'Contacte o administrador de seu sistema para lhe disponibilize as atualizações necessárias',
  contentTypeUpdateAvailable: 'Atualização disponível',
  contentTypeRestricted: 'Tipo de conteúdo restrito',
  contentTypeRestrictedDesc: 'O uso deste tipo de conteúdo foi restringido pelo administrador.',
  contentTypeNotInstalled: 'Tipo de conteúdo não instalado',
  contentTypeNotInstalledDesc: 'Não tem permissão para instalar tipos de conteúdo.',
  theContentType: 'o tipo de conteúdo',
  currentMenuSelected: 'seleção atual',
  errorCommunicatingHubTitle: 'Não foi possível comunicar com o Hub.',
  errorCommunicatingHubContent: 'Ocorreu um erro. Por favor, tente novamente.',
  warningNoContentTypesInstalled: 'Nenhum tipo de conteúdo está instalado.',
  warningChangeBrowsingToSeeResults:
    'Clique em <em>todos</em> para obter a lista de todos os conteúdos que pode instalar.',
  warningUpdateAvailableTitle: 'Está disponível uma nova versão de :contentType.',
  warningUpdateAvailableBody: 'Atualize para a última versão para uma melhor experiência.',
  licenseDescription:
    'Algumas das características desta licença são indicadas abaixo. Clique no ícone de informações acima para ler o texto original da licença.',
  licenseModalTitle: 'Detalhes da licença',
  licenseModalSubtitle: 'Selecione uma licença para ver informações sobre o uso apropriado',
  licenseUnspecified: 'Não especificado',
  licenseCanUseCommercially: 'Pode usar comercialmente',
  licenseCanModify: 'Pode modificar',
  licenseCanDistribute: 'Pode distribuir',
  licenseCanSublicense: 'Pode sub-licenciar',
  licenseCanHoldLiable: 'Pode ser responsabilizado',
  licenseCannotHoldLiable: 'Não pode ser responsabilizado',
  licenseMustIncludeCopyright: 'Obrigatório incluir direitos de autor',
  licenseMustIncludeLicense: 'Obrigatório incluir licença',
  licenseFetchDetailsFailed: 'Ocorreu um erro na obtenção dos detalhes da licença',
  imageLightboxTitle: 'Imagens',
  imageLightBoxProgress: ':num de :total',
  nextImage: 'Imagem seguinte',
  previousImage: 'Imagem anterior',
  screenshots: 'Capturas de ecrã',
  reloadButtonLabel: 'Recarregar',
  videoQuality: 'Etiqueta da qualidade de vídeo',
  videoQualityDescription:
    'Esta etiqueta ajuda o utilizador a identificar a qualidade atual do vídeo. Ex.: 1080p, 720p, HD ou Mobile',
  videoQualityDefaultLabel: 'Qualidade :index',
  noContentTypesAvailable: 'Nenhum tipo de conteúdo está disponível',
  noContentTypesAvailableDesc:
    'O seu site está a ter dificuldades na ligação a H5P.org para obter a listagem os tipos de conteúdo disponíveis.',
  contentTypeCacheOutdated: 'Lista de tipos de conteúdo desatualizada',
  contentTypeCacheOutdatedDesc:
    'O seu site está a ter dificuldades na ligação a H5P.org para verificar a existência de atualizações disponíveis. Pode não ser capaz de atualizar ou instalar novos tipos de conteúdo.',
  tryAgain: 'Tentar novamente',
  getHelp: 'Ajuda',
  untitled: 'Sem título :libraryTitle',
  title: 'Título',
  metadata: 'Metadados',
  addTitle: 'Adicionar título',
  usedForSearchingReportsAndCopyrightInformation: 'Usado para pesquisas, relatórios e informações de direitos de autor',
  metadataSharingAndLicensingInfo: 'Metadados (informações de partilha e licenciamento)',
  fillInTheFieldsBelow: 'Preencha os campos abaixo',
  saveMetadata: 'Guardar metadados',
  addAuthor: 'Guardar autor',
  confirmRemoveAuthor: 'Tem a certeza de que pretende remover este autor?',
  addNewChange: 'Adicionar nova alteração',
  confirmDeleteChangeLog: 'Tem a certeza de que pretende excluir esta entrada do registo de alterações?',
  changelogDescription:
    'Algumas licenças exigem que sejam registadas e exibidas as alterações feitas no trabalho original ou derivados. Pode registar aqui as suas alterações por motivos de licenciamento ou apenas para permitir que possa, tal como outras pessoas, acompanhar as alterações feitas neste conteúdo.',
  logThisChange: 'Registar alteração',
  newChangeHasBeenLogged: 'A nova alteração foi guardada no registo',
  loggedChanges: 'Alterações registadas',
  noChangesHaveBeenLogged: 'Nenhuma alteração foi registada',
  errorHeader: 'Ocorreu um erro',
  errorCalculatingMaxScore:
    'Não foi possível calcular a pontuação máxima para este conteúdo. A pontuação máxima é assumida como sendo 0. Se achar que não está correto, contacte o seu administrador.',
  maxScoreSemanticsMissing: 'Não foi possível encontrar a semântica esperada no conteúdo.',
  copyButton: 'Copiar',
  copiedButton: 'Copiado',
  pasteButton: 'Colar',
  pasteAndReplaceButton: 'Colar & Substituir',
  pasteContent: 'Substituir conteúdo',
  confirmPasteContent:
    'Se continuar, irá substituir o conteúdo atual pelo conteúdo da área de transferência. O conteúdo atual será perdido. Tem a certeza de que pretende continuar?',
  confirmPasteButtonText: 'Substituir conteúdo',
  copyToClipboard: 'Copiar conteúdo H5P para a área de transferência',
  copiedToClipboard: 'O conteúdo foi copiado para a área de transferência',
  pasteFromClipboard: 'Colar conteúdo H5P da área de transferência',
  pasteAndReplaceFromClipboard: 'Substituir o conteúdo existente pelo conteúdo H5P da área de transferência',
  pasteNoContent: 'Nenhum conteúdo H5P na área de transferência',
  pasteError: 'Não foi possível colar',
  pasteContentNotSupported: 'O conteúdo que está a tentar colar não é suportado neste contexto',
  pasteContentRestricted: 'The content in the clipboard has been restricted on this site',
  pasteTooOld:
    'O conteúdo que está a tentar colar é de uma versão inferior (:clip) à versão suportada neste contexto (:local). Se possível, tente atualizar o conteúdo que deseja colar, copie-o novamente e tente colá-lo aqui.',
  pasteTooNew:
    'O conteúdo que está a tentar colar é de uma versão superior (:clip) à versão suportada neste contexto (:local). Se possível, tente atualizar este conteúdo, e depois tente colar novamente.',
  ok: 'OK',
  avTablistLabel: 'Inserir usando',
  tabTitleBasicFileUpload: 'Carregar ficheiro',
  tabTitleInputLinkURL: 'Hiperligação/URL',
  errorTooHighVersion: 'Os parâmetros contêm %used mas apenas são suportados %supported ou anterior.',
  errorNotSupported: 'Os parâmetros contêm %used que não são suportados.',
  errorParamsBroken: 'Os parâmetros estão corrompidos.',
  libraryMissing: 'Está em falta a livraria obrigatória %lib.',
  scriptMissing: 'Não foi possível carregar o script de actualizações de %lib.',
  language: 'Idioma',
  noLanguagesSupported: 'Nenhum idioma suportado',
  changeLanguage: 'Alterar idioma para :language?',
  thisWillPotentially:
    'Isto redefinirá potencialmente todos os textos e traduções. Não pode desfazer esta ação. O conteúdo em si não será alterado. Pretende prosseguir?',
  notAllTextsChanged: 'Nem todos os textos foram alterados. Existe apenas uma tradução parcial para :language.',
  contributeTranslations:
    'Se pretende completar a tradução para :language, clique para obter informações sobre <a href=":url" target="_new">contribuir com traduções para H5P</a>',
  unknownLibrary: "Infelizmente, o tipo de conteúdo selecionado '%lib' não está instalado neste sistema.",
  proceedButtonLabel: 'Prosseguir para guardar',
  enterFullscreenButtonLabel: 'Mostrar em ecrã inteiro',
  exitFullscreenButtonLabel: 'Sair de ecrã inteiro',
  a11yTitleShowLabel: 'Show label for AT',
  a11yTitleHideLabel: 'Hide label for AT',
  reuseSuccess: ':title was successfully imported from the H5P Hub.',
  noContentHeader: 'No Fitting Content?',
  noContentSuggestion: 'Create one yourself!',
  tutorials: 'Tutorials',
  contentSectionAll: 'All shared content',
  popularContent: 'Popular Content',
  allPopular: 'All Popular',
  newOnTheHub: 'New on the Hub',
  allNew: 'All New',
  filterBy: 'Filter by',
  filter: 'Filter',
  filters: {
    level: {
      dropdownLabel: 'Level',
      dialogHeader: 'Select level of education',
      dialogButtonLabel: 'Filter level of education',
    },
    language: {
      dropdownLabel: 'Language',
      dialogHeader: 'Select language(s)',
      dialogButtonLabel: 'Filter languages',
      searchPlaceholder: 'Type to search for languages',
    },
    reviewed: {
      dropdownLabel: 'Reviewed',
      dialogHeader: 'Reviewed Content',
      dialogButtonLabel: 'Filter',
      optionLabel: 'Show only reviewed content',
    },
    contentTypes: {
      dropdownLabel: 'Content types',
      dialogHeader: 'Select Content type(s)',
      dialogButtonLabel: 'Filter Content Types',
      searchPlaceholder: 'Type to search for content types',
    },
    disciplines: {
      dropdownLabel: 'Discipline',
      dialogHeader: 'Select Your Discipline',
      dialogButtonLabel: 'Filter Disciplines',
      searchPlaceholder: 'Type to search for disciplines',
    },
    licenses: {
      dropdownLabel: 'License',
      dialogHeader: 'Select preferred rights of use',
      dialogButtonLabel: 'Filter licenses',
      options: { modified: 'Can be modified', commercial: 'Allows commercial use' },
    },
  },
  clearFilters: 'Clear all filters',
  contentSearchFieldPlaceholder: 'Search for Content',
  loadingContentTitle: 'We are loading content for you...',
  loadingContentSubtitle: 'Please wait',
  by: 'By',
  dropdownButton: 'Open dropdown',
  paginationNavigation: 'Pagination navigation',
  page: 'Page',
  currentPage: 'Current page',
  nextPage: 'Go to next page',
  previousPage: 'Go to previous page',
  contentPreviewButtonLabel: 'Preview',
  contentDownloadButtonLabel: 'Get Content',
  reuseContentTabLabel: 'Get Shared Content',
  contentPublisherPanelHeader: 'Publisher Info',
  noContentFoundDesc: 'There is no content that matches your search criteria.',
  h5pType: 'H5P Type',
  level: 'Level',
  size: 'Size',
  failedFetchingData: 'Failed fetching data',
  filterErrorMessage: 'Something went wrong. Please reload the page.',
  in: 'in',
  navigateToParent: 'Navigate to parent',
};
