{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/insert_image_modal_insert

    Insert image body template.

    Example context (json):
    {
        "showdropzone": true,
        "showfilepicker": true
    }

}}
<div class="tiny_image_insert_image">
    <div class="tiny_image_dropzone d-flex flex-column justify-content-center">
        {{#showdropzone}}
            <div class="tiny_image_dropzone_container"></div>
            <!-- File input -->
            <input type="file" id="tiny_image_fileinput" accept="image/*" class="d-none">
        {{/showdropzone}}
        {{^showdropzone}}
            <div class="text-center">
                <!-- Dropzone icon -->
                <i class="fa-6x pb-2 text-secondary fa fa-cloud"></i>
                <!-- Dropzone string -->
                <div class="lead text-center">
                    <p class="mb-0">
                    {{#showfilepicker}}
                        {{#str}} repositoryuploadnotpermitted, tiny_media {{/str}}</p>
                    {{/showfilepicker}}
                    {{^showfilepicker}}
                        {{#str}} repositorynotpermitted, tiny_media {{/str}}
                    {{/showfilepicker}}
                </div>
            </div>
        {{/showdropzone}}
    </div>
</div>
