<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Tiny text editor integration.
 *
 * @package    editor_tiny
 * @copyright  2021 Andrew Lyons <andrew@nicols.co.uk>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['pluginname'] = 'TinyMCE editor';
$string['settings'] = 'General settings';
$string['privacy:reason'] = 'The TinyMCE editor does not store any preferences or user data.';
$string['branding'] = 'TinyMCE branding';
$string['branding_desc'] = 'Support TinyMCE by displaying the logo in the bottom corner of the text editor. The logo links to the TinyMCE website.';
$string['extended_valid_elements'] = 'Extended valid elements';
$string['extended_valid_elements_desc'] = 'This allows you to use additional HTML elements and attributes in the editor.';
$string['plugin_enabled'] = '{$a} enabled.';
$string['plugin_disabled'] = '{$a} disabled.';
$string['subplugintype_tiny'] = 'TinyMCE plugin';
$string['subplugintype_tiny_plural'] = 'TinyMCE plugins';
$string['tiny:hash'] = '#';
$string['tiny:accessibility'] = 'Accessibility';
$string['tiny:accordion'] = 'Accordion';
$string['tiny:accordion_body...'] = 'Accordion body...';
$string['tiny:accordion_summary...'] = 'Accordion summary...';
$string['tiny:action'] = 'Action';
$string['tiny:activity'] = 'Activity';
$string['tiny:address'] = 'Address';
$string['tiny:advanced'] = 'Advanced';
$string['tiny:align'] = 'Align';
$string['tiny:align_center'] = 'Align centre';
$string['tiny:align_left'] = 'Align left';
$string['tiny:align_right'] = 'Align right';
$string['tiny:alignment'] = 'Alignment';
$string['tiny:alignment_0'] = 'Alignment {0}';
$string['tiny:all'] = 'All';
$string['tiny:alternative_description'] = 'Alternative description';
$string['tiny:alternative_source'] = 'Alternative source';
$string['tiny:alternative_source_url'] = 'Alternative source URL';
$string['tiny:anchor'] = 'Anchor';
$string['tiny:anchor...'] = 'Anchor...';
$string['tiny:anchors'] = 'Anchors';
$string['tiny:animals_and_nature'] = 'Animals and nature';
$string['tiny:arrows'] = 'Arrows';
$string['tiny:b'] = 'B';
$string['tiny:background_color'] = 'Background colour';
$string['tiny:background_color_0'] = 'Background colour {0}';
$string['tiny:black'] = 'Black';
$string['tiny:block'] = 'Block';
$string['tiny:block_0'] = 'Block {0}';
$string['tiny:blockquote'] = 'Blockquote';
$string['tiny:blocks'] = 'Paragraph styles';
$string['tiny:blue'] = 'Blue';
$string['tiny:blue_component'] = 'Blue component';
$string['tiny:body'] = 'Body';
$string['tiny:bold'] = 'Bold';
$string['tiny:border'] = 'Border';
$string['tiny:border_color'] = 'Border colour';
$string['tiny:border_style'] = 'Border style';
$string['tiny:border_width'] = 'Border width';
$string['tiny:bottom'] = 'Bottom';
$string['tiny:browse_files'] = 'Browse files';
$string['tiny:browse_for_an_image'] = 'Browse for an image';
$string['tiny:browse_links'] = 'Browse links';
$string['tiny:bullet_list'] = 'Bullet list';
$string['tiny:cancel'] = 'Cancel';
$string['tiny:caption'] = 'Caption';
$string['tiny:cell'] = 'Cell';
$string['tiny:cell_padding'] = 'Cell padding';
$string['tiny:cell_properties'] = 'Cell properties';
$string['tiny:cell_spacing'] = 'Cell spacing';
$string['tiny:cell_styles'] = 'Cell styles';
$string['tiny:cell_type'] = 'Cell type';
$string['tiny:center'] = 'Centre';
$string['tiny:characters'] = 'Characters';
$string['tiny:characters_no_spaces'] = 'Characters (no spaces)';
$string['tiny:circle'] = 'Circle';
$string['tiny:class'] = 'Class';
$string['tiny:clear_formatting'] = 'Clear formatting';
$string['tiny:close'] = 'Close';
$string['tiny:code'] = 'Code';
$string['tiny:code_sample...'] = 'Code sample...';
$string['tiny:code_view'] = 'Code view';
$string['tiny:color_picker'] = 'Colour picker';
$string['tiny:color_swatch'] = 'Colour swatch';
$string['tiny:cols'] = 'Cols';
$string['tiny:column'] = 'Column';
$string['tiny:column_clipboard_actions'] = 'Column clipboard actions';
$string['tiny:column_group'] = 'Column group';
$string['tiny:column_header'] = 'Column header';
$string['tiny:constrain_proportions'] = 'Constrain proportions';
$string['tiny:copy'] = 'Copy';
$string['tiny:copy_column'] = 'Copy column';
$string['tiny:copy_row'] = 'Copy row';
$string['tiny:could_not_find_the_specified_string.'] = 'Could not find the specified string.';
$string['tiny:could_not_load_emojis'] = 'Could not load emojis';
$string['tiny:count'] = 'Count';
$string['tiny:currency'] = 'Currency';
$string['tiny:current_window'] = 'Current window';
$string['tiny:custom_color'] = 'Custom colour';
$string['tiny:custom...'] = 'Custom...';
$string['tiny:cut'] = 'Cut';
$string['tiny:cut_column'] = 'Cut column';
$string['tiny:cut_row'] = 'Cut row';
$string['tiny:dark_blue'] = 'Dark blue';
$string['tiny:dark_gray'] = 'Dark grey';
$string['tiny:dark_green'] = 'Dark green';
$string['tiny:dark_orange'] = 'Dark orange';
$string['tiny:dark_purple'] = 'Dark purple';
$string['tiny:dark_red'] = 'Dark red';
$string['tiny:dark_turquoise'] = 'Dark turquoise';
$string['tiny:dark_yellow'] = 'Dark yellow';
$string['tiny:dashed'] = 'Dashed';
$string['tiny:datetime'] = 'Date/time';
$string['tiny:decrease_indent'] = 'Decrease indent';
$string['tiny:default'] = 'Default';
$string['tiny:delete_accordion'] = 'Delete accordion';
$string['tiny:delete_column'] = 'Delete column';
$string['tiny:delete_row'] = 'Delete row';
$string['tiny:delete_table'] = 'Delete table';
$string['tiny:dimensions'] = 'Dimensions';
$string['tiny:disc'] = 'Disc';
$string['tiny:div'] = 'Div';
$string['tiny:document'] = 'Document';
$string['tiny:dotted'] = 'Dotted';
$string['tiny:double'] = 'Double';
$string['tiny:drop_an_image_here'] = 'Drop an image here';
$string['tiny:dropped_file_type_is_not_supported'] = 'Dropped file type is not supported';
$string['tiny:edit'] = 'Edit';
$string['tiny:embed'] = 'Embed';
$string['tiny:emojis'] = 'Emojis';
$string['tiny:emojis...'] = 'Emojis...';
$string['tiny:error'] = 'Error';
$string['tiny:error_form_submit_field_collision.'] = 'Error: Form submit field collision.';
$string['tiny:error_no_form_element_found.'] = 'Error: No form element found.';
$string['tiny:extended_latin'] = 'Extended Latin';
$string['tiny:failed_to_initialize_plugin_0'] = 'Failed to initialize plugin: {0}';
$string['tiny:failed_to_load_plugin_url_0'] = 'Failed to load plugin url: {0}';
$string['tiny:failed_to_load_plugin_0_from_url_1'] = 'Failed to load plugin: {0} from url {1}';
$string['tiny:failed_to_upload_image_0'] = 'Failed to upload image: {0}';
$string['tiny:file'] = 'File';
$string['tiny:find'] = 'Find';
$string['tiny:find_and_replace'] = 'Find and replace';
$string['tiny:find_if_searchreplace_plugin_activated'] = 'Find (if search and replace plugin activated)';
$string['tiny:find_and_replace...'] = 'Find and replace...';
$string['tiny:find_in_selection'] = 'Find in selection';
$string['tiny:find_whole_words_only'] = 'Find whole words only';
$string['tiny:flags'] = 'Flags';
$string['tiny:focus_to_contextual_toolbar'] = 'Focus to contextual toolbar';
$string['tiny:focus_to_element_path'] = 'Focus to element path';
$string['tiny:focus_to_menubar'] = 'Focus to menubar';
$string['tiny:focus_to_toolbar'] = 'Focus to toolbar';
$string['tiny:font'] = 'Font';
$string['tiny:font_size_0'] = 'Font size {0}';
$string['tiny:font_sizes'] = 'Font sizes';
$string['tiny:font_0'] = 'Font {0}';
$string['tiny:fonts'] = 'Fonts';
$string['tiny:food_and_drink'] = 'Food and drink';
$string['tiny:footer'] = 'Footer';
$string['tiny:format'] = 'Format';
$string['tiny:format_0'] = 'Format {0}';
$string['tiny:formats'] = 'Formats';
$string['tiny:fullscreen'] = 'Fullscreen';
$string['tiny:g'] = 'G';
$string['tiny:general'] = 'General';
$string['tiny:gray'] = 'Grey';
$string['tiny:green'] = 'Green';
$string['tiny:green_component'] = 'Green component';
$string['tiny:groove'] = 'Groove';
$string['tiny:handy_shortcuts'] = 'Handy shortcuts';
$string['tiny:header'] = 'Header';
$string['tiny:header_cell'] = 'Header cell';
$string['tiny:heading_1'] = 'Heading 1';
$string['tiny:heading_2'] = 'Heading 2';
$string['tiny:heading_3'] = 'Heading 3';
$string['tiny:heading_4'] = 'Heading 4';
$string['tiny:heading_5'] = 'Heading 5';
$string['tiny:heading_6'] = 'Heading 6';
$string['tiny:headings'] = 'Headings';
$string['tiny:height'] = 'Height';
$string['tiny:help'] = 'Help';
$string['tiny:hex_color_code'] = 'Hex colour code';
$string['tiny:hidden'] = 'Hidden';
$string['tiny:horizontal_align'] = 'Horizontal align';
$string['tiny:horizontal_line'] = 'Horizontal line';
$string['tiny:horizontal_space'] = 'Horizontal space';
$string['tiny:id'] = 'ID';
$string['tiny:id_should_start_with_a_letter_followed_only_by_letters_numbers_dashes_dots_colons_or_'] = 'ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.';
$string['tiny:image_is_decorative'] = 'Image is decorative';
$string['tiny:image_list'] = 'Image list';
$string['tiny:image_title'] = 'Image title';
$string['tiny:image...'] = 'Image...';
$string['tiny:imageproxy_http_error_could_not_find_image_proxy'] = 'ImageProxy HTTP error: Could not find Image Proxy';
$string['tiny:imageproxy_http_error_incorrect_image_proxy_url'] = 'ImageProxy HTTP error: Incorrect Image Proxy URL';
$string['tiny:imageproxy_http_error_rejected_request'] = 'ImageProxy HTTP error: Rejected request';
$string['tiny:imageproxy_http_error_unknown_imageproxy_error'] = 'ImageProxy HTTP error: Unknown ImageProxy error';
$string['tiny:increase_indent'] = 'Increase indent';
$string['tiny:inline'] = 'Inline';
$string['tiny:insert'] = 'Insert';
$string['tiny:insert_template'] = 'Insert template';
$string['tiny:insert_accordion'] = 'Insert accordion';
$string['tiny:insert_column_after'] = 'Insert column after';
$string['tiny:insert_column_before'] = 'Insert column before';
$string['tiny:insert_datetime'] = 'Insert date/time';
$string['tiny:insert_image'] = 'Insert image';
$string['tiny:insert_link_if_link_plugin_activated'] = 'Insert link (if link plugin activated)';
$string['tiny:insert_row_after'] = 'Insert row after';
$string['tiny:insert_row_before'] = 'Insert row before';
$string['tiny:insert_table'] = 'Insert table';
$string['tiny:insert_template...'] = 'Insert template...';
$string['tiny:insert_video'] = 'Insert video';
$string['tiny:insertedit_code_sample'] = 'Insert/edit code sample';
$string['tiny:insertedit_image'] = 'Insert/edit image';
$string['tiny:insertedit_link'] = 'Insert/edit link';
$string['tiny:insertedit_media'] = 'Insert/edit media';
$string['tiny:insertedit_video'] = 'Insert/edit video';
$string['tiny:inset'] = 'Inset';
$string['tiny:invalid_hex_color_code_0'] = 'Invalid hex colour code: {0}';
$string['tiny:invalid_input'] = 'Invalid input';
$string['tiny:italic'] = 'Italic';
$string['tiny:justify'] = 'Justify';
$string['tiny:keyboard_navigation'] = 'Keyboard navigation';
$string['tiny:language'] = 'Language';
$string['tiny:learn_more...'] = 'Learn more...';
$string['tiny:left'] = 'Left';
$string['tiny:left_to_right'] = 'Left to right';
$string['tiny:light_blue'] = 'Light blue';
$string['tiny:light_gray'] = 'Light grey';
$string['tiny:light_green'] = 'Light green';
$string['tiny:light_purple'] = 'Light purple';
$string['tiny:light_red'] = 'Light red';
$string['tiny:light_yellow'] = 'Light yellow';
$string['tiny:line_height'] = 'Line height';
$string['tiny:link_list'] = 'Link list';
$string['tiny:link...'] = 'Link...';
$string['tiny:list_properties'] = 'List properties';
$string['tiny:list_properties...'] = 'List properties...';
$string['tiny:loading_emojis...'] = 'Loading emojis...';
$string['tiny:loading...'] = 'Loading...';
$string['tiny:lower_alpha'] = 'Lower Alpha';
$string['tiny:lower_greek'] = 'Lower Greek';
$string['tiny:lower_roman'] = 'Lower Roman';
$string['tiny:match_case'] = 'Match case';
$string['tiny:mathematical'] = 'Mathematical';
$string['tiny:media_poster_image_url'] = 'Media poster (Image URL)';
$string['tiny:media...'] = 'Media...';
$string['tiny:medium_blue'] = 'Medium blue';
$string['tiny:medium_gray'] = 'Medium grey';
$string['tiny:medium_purple'] = 'Medium purple';
$string['tiny:merge_cells'] = 'Merge cells';
$string['tiny:middle'] = 'Middle';
$string['tiny:midnight_blue'] = 'Midnight blue';
$string['tiny:more...'] = 'More...';
$string['tiny:name'] = 'Name';
$string['tiny:navy_blue'] = 'Navy blue';
$string['tiny:new_document'] = 'New document';
$string['tiny:new_window'] = 'New window';
$string['tiny:next'] = 'Next';
$string['tiny:no'] = 'No';
$string['tiny:no_alignment'] = 'No alignment';
$string['tiny:no_color'] = 'No colour';
$string['tiny:nonbreaking_space'] = 'Nonbreaking space';
$string['tiny:none'] = 'None';
$string['tiny:numbered_list'] = 'Numbered list';
$string['tiny:or'] = 'OR';
$string['tiny:objects'] = 'Objects';
$string['tiny:ok'] = 'Ok';
$string['tiny:open_help_dialog'] = 'Open help dialog';
$string['tiny:open_link'] = 'Open link';
$string['tiny:open_link_in...'] = 'Open link in...';
$string['tiny:open_popup_menu_for_split_buttons'] = 'Open popup menu for split buttons';
$string['tiny:orange'] = 'Orange';
$string['tiny:outset'] = 'Outset';
$string['tiny:page_break'] = 'Page break';
$string['tiny:paragraph'] = 'Paragraph';
$string['tiny:paste'] = 'Paste';
$string['tiny:paste_as_text'] = 'Paste as text';
$string['tiny:paste_column_after'] = 'Paste column after';
$string['tiny:paste_column_before'] = 'Paste column before';
$string['tiny:paste_is_now_in_plain_text_mode._contents_will_now_be_pasted_as_plain_text_until_you_'] = 'Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.';
$string['tiny:paste_or_type_a_link'] = 'Paste or type a link';
$string['tiny:paste_row_after'] = 'Paste row after';
$string['tiny:paste_row_before'] = 'Paste row before';
$string['tiny:paste_your_embed_code_below'] = 'Paste your embed code below:';
$string['tiny:people'] = 'People';
$string['tiny:plugins'] = 'Plugins';
$string['tiny:plugins_installed_0'] = 'Plugins installed ({0}):';
$string['tiny:powered_by_0'] = 'Powered by {0}';
$string['tiny:pre'] = 'Pre';
$string['tiny:preferences'] = 'Preferences';
$string['tiny:preformatted'] = 'Preformatted';
$string['tiny:premium_plugins'] = 'Premium plugins:';
$string['tiny:press_the_up_and_down_arrow_keys_to_resize_the_editor.'] = 'Press the Up and Down arrow keys to resize the editor.';
$string['tiny:press_the_arrow_keys_to_resize_the_editor.'] = 'Press the arrow keys to resize the editor.';
$string['tiny:press_0_for_help'] = 'Press {0} for help';
$string['tiny:preview'] = 'Preview';
$string['tiny:previous'] = 'Previous';
$string['tiny:print'] = 'Print';
$string['tiny:print...'] = 'Print...';
$string['tiny:purple'] = 'Purple';
$string['tiny:quotations'] = 'Quotations';
$string['tiny:r'] = 'R';
$string['tiny:range_0_to_255'] = 'Range 0 to 255';
$string['tiny:red'] = 'Red';
$string['tiny:red_component'] = 'Red component';
$string['tiny:redo'] = 'Redo';
$string['tiny:remove'] = 'Remove';
$string['tiny:remove_color'] = 'Remove colour';
$string['tiny:remove_link'] = 'Remove link';
$string['tiny:replace'] = 'Replace';
$string['tiny:replace_all'] = 'Replace all';
$string['tiny:replace_with'] = 'Replace with';
$string['tiny:resize'] = 'Resize';
$string['tiny:restore_last_draft'] = 'Restore last draft';
$string['tiny:reveal_or_hide_additional_toolbar_items'] = 'Reveal or hide additional toolbar items';
$string['tiny:rich_text_area'] = 'Rich text area';
$string['tiny:rich_text_area._press_alt-0_for_help.'] = 'Rich text area. Press ALT-0 for help.';
$string['tiny:rich_text_area._press_alt-f9_for_menu._press_alt-f10_for_toolbar._press_alt-0_for_hel'] = 'Rich text area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help.';
$string['tiny:ridge'] = 'Ridge';
$string['tiny:right'] = 'Right';
$string['tiny:right_to_left'] = 'Right to left';
$string['tiny:row'] = 'Row';
$string['tiny:row_clipboard_actions'] = 'Row clipboard actions';
$string['tiny:row_group'] = 'Row group';
$string['tiny:row_header'] = 'Row header';
$string['tiny:row_properties'] = 'Row properties';
$string['tiny:row_type'] = 'Row type';
$string['tiny:rows'] = 'Rows';
$string['tiny:save'] = 'Save';
$string['tiny:save_if_save_plugin_activated'] = 'Save (if save plugin activated)';
$string['tiny:scope'] = 'Scope';
$string['tiny:search'] = 'Search';
$string['tiny:select_all'] = 'Select all';
$string['tiny:select...'] = 'Select...';
$string['tiny:selection'] = 'Selection';
$string['tiny:shortcut'] = 'Shortcut';
$string['tiny:show_blocks'] = 'Show paragraph styles';
$string['tiny:show_caption'] = 'Show caption';
$string['tiny:show_invisible_characters'] = 'Show invisible characters';
$string['tiny:size'] = 'Size';
$string['tiny:solid'] = 'Solid';
$string['tiny:source'] = 'Source';
$string['tiny:source_code'] = 'Source code';
$string['tiny:special_character'] = 'Special character';
$string['tiny:special_character...'] = 'Special character...';
$string['tiny:split_cell'] = 'Split cell';
$string['tiny:square'] = 'Square';
$string['tiny:start_list_at_number'] = 'Start list at number';
$string['tiny:strikethrough'] = 'Strikethrough';
$string['tiny:style'] = 'Style';
$string['tiny:subscript'] = 'Subscript';
$string['tiny:superscript'] = 'Superscript';
$string['tiny:switch_to_or_from_fullscreen_mode'] = 'Switch to or from fullscreen mode';
$string['tiny:symbols'] = 'Symbols';
$string['tiny:system_font'] = 'System font';
$string['tiny:table'] = 'Table';
$string['tiny:table_caption'] = 'Table caption';
$string['tiny:table_properties'] = 'Table properties';
$string['tiny:table_styles'] = 'Table styles';
$string['tiny:template'] = 'Template';
$string['tiny:templates'] = 'Templates';
$string['tiny:text'] = 'Text';
$string['tiny:text_color'] = 'Text colour';
$string['tiny:text_color_0'] = 'Text color {0}';
$string['tiny:text_to_display'] = 'Text to display';
$string['tiny:the_url_you_entered_seems_to_be_an_email_address._do_you_want_to_add_the_required_mai'] = 'The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?';
$string['tiny:the_url_you_entered_seems_to_be_an_external_link._do_you_want_to_add_the_required_htt'] = 'The URL you entered seems to be an external link. Do you want to add the required http:// prefix?';
$string['tiny:the_url_you_entered_seems_to_be_an_external_link._do_you_want_to_add_the_required_ht1'] = 'The URL you entered seems to be an external link. Do you want to add the required https:// prefix?';
$string['tiny:title'] = 'Title';
$string['tiny:to_open_the_popup_press_shiftenter'] = 'To open the popup, press Shift+Enter';
$string['tiny:toggle_accordion'] = 'Toggle accordion';
$string['tiny:tools'] = 'Tools';
$string['tiny:top'] = 'Top';
$string['tiny:travel_and_places'] = 'Travel and places';
$string['tiny:turquoise'] = 'Turquoise';
$string['tiny:underline'] = 'Underline';
$string['tiny:undo'] = 'Undo';
$string['tiny:upload'] = 'Upload';
$string['tiny:uploading_image'] = 'Uploading image';
$string['tiny:upper_alpha'] = 'Upper Alpha';
$string['tiny:upper_roman'] = 'Upper Roman';
$string['tiny:url'] = 'Url';
$string['tiny:user_defined'] = 'User defined';
$string['tiny:valid'] = 'Valid';
$string['tiny:version'] = 'Version';
$string['tiny:vertical_align'] = 'Vertical align';
$string['tiny:vertical_space'] = 'Vertical space';
$string['tiny:view'] = 'View';
$string['tiny:visual_aids'] = 'Visual aids';
$string['tiny:warn'] = 'Warn';
$string['tiny:white'] = 'White';
$string['tiny:width'] = 'Width';
$string['tiny:word_count'] = 'Word count';
$string['tiny:words'] = 'Words';
$string['tiny:words_0'] = 'Words: {0}';
$string['tiny:yellow'] = 'Yellow';
$string['tiny:yes'] = 'Yes';
$string['tiny:you_are_using_0'] = 'You are using {0}';
$string['tiny:you_have_unsaved_changes_are_you_sure_you_want_to_navigate_away'] = 'You have unsaved changes. Are you sure you want to navigate away?';
$string['tiny:your_browser_doesnt_support_direct_access_to_the_clipboard._please_use_the_ctrlxcv_ke'] = 'Your browser doesn\'t support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.';
$string['tiny:alignment1'] = 'alignment';
$string['tiny:austral_sign'] = 'austral sign';
$string['tiny:cedi_sign'] = 'cedi sign';
$string['tiny:colon_sign'] = 'colon sign';
$string['tiny:cruzeiro_sign'] = 'cruzeiro sign';
$string['tiny:currency_sign'] = 'currency sign';
$string['tiny:dollar_sign'] = 'dollar sign';
$string['tiny:dong_sign'] = 'dong sign';
$string['tiny:drachma_sign'] = 'drachma sign';
$string['tiny:euro-currency_sign'] = 'euro-currency sign';
$string['tiny:example'] = 'example';
$string['tiny:formatting'] = 'formatting';
$string['tiny:french_franc_sign'] = 'french franc sign';
$string['tiny:german_penny_symbol'] = 'german penny symbol';
$string['tiny:guarani_sign'] = 'guarani sign';
$string['tiny:history'] = 'history';
$string['tiny:hryvnia_sign'] = 'hryvnia sign';
$string['tiny:indentation'] = 'indentation';
$string['tiny:indian_rupee_sign'] = 'indian rupee sign';
$string['tiny:kip_sign'] = 'kip sign';
$string['tiny:lira_sign'] = 'lira sign';
$string['tiny:livre_tournois_sign'] = 'livre tournois sign';
$string['tiny:manat_sign'] = 'manat sign';
$string['tiny:mill_sign'] = 'mill sign';
$string['tiny:naira_sign'] = 'naira sign';
$string['tiny:new_sheqel_sign'] = 'new sheqel sign';
$string['tiny:nordic_mark_sign'] = 'nordic mark sign';
$string['tiny:peseta_sign'] = 'peseta sign';
$string['tiny:peso_sign'] = 'peso sign';
$string['tiny:ruble_sign'] = 'ruble sign';
$string['tiny:rupee_sign'] = 'rupee sign';
$string['tiny:spesmilo_sign'] = 'spesmilo sign';
$string['tiny:styles'] = 'styles';
$string['tiny:tenge_sign'] = 'tenge sign';
$string['tiny:tugrik_sign'] = 'tugrik sign';
$string['tiny:turkish_lira_sign'] = 'turkish lira sign';
$string['tiny:won_sign'] = 'won sign';
$string['tiny:yen_character'] = 'yen character';
$string['tiny:yenyuan_character_variant_one'] = 'yen/yuan character variant one';
$string['tiny:yuan_character'] = 'yuan character';
$string['tiny:yuan_character_in_hong_kong_and_taiwan'] = 'yuan character, in hong kong and taiwan';
$string['tiny:0_characters'] = '{0} characters';
$string['tiny:0_columns_1_rows'] = '{0} columns, {1} rows';
$string['tiny:0_words'] = '{0} words';
