{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/embed/body/insert_media_body

    Insert media body template.

    Example context (json):
    {
        "showDropzone": true,
        "showFilePicker": true
    }

}}
<div class="tiny_media_insert_media" id="tiny-insert-media">
    <div class="d-flex flex-column justify-content-center">
        {{#showDropzone}}
            <div class="tiny_media_dropzone_container"></div>
        {{/showDropzone}}
        {{^showDropzone}}
            <div class="text-center">
                <i class="fa-6x pb-2 text-secondary fa fa-cloud"></i>
                <div class="lead text-center">
                    <p class="mb-0">
                        {{#showFilePicker}}
                            {{#str}} nofilepicker, tiny_media {{/str}}
                        {{/showFilePicker}}
                        {{^showFilePicker}}
                            {{#str}} nofilepickerrepository, tiny_media, {{fileType}} {{/str}}
                        {{/showFilePicker}}
                    </p>
                </div>
            </div>
        {{/showDropzone}}
    </div>
</div>
