{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/embed/body/media_thumbnail_body

    Insert thumbnail preview body template.

    Example context (json):
    {

    }

}}
<div class="container tiny_media_insert_media d-none" id="tiny-media-thumbnail-preview">
    <div class="row">
        <div class="tiny_media_preview_col col-lg-12 p-0">
            <div class="tiny_media_preview_box border rounded">
                <button class="btn rounded-circle tiny_media_delete_icon delete_tiny_thumbnail" tabindex="0" title="{{#str}} deletemediathumbnail, tiny_media {{/str}}">
                    <i class="fa fa-trash-o"></i>
                </button>
                <img class="tiny_media_thumbnail_preview img-fluid" alt="media thumbnail image" src="data:,">
            </div>
        </div>
    </div>
</div>
