.mce-content-body .h5p-placeholder + br {
    display: none;
}
.mce-content-body .h5p-placeholder {
    color: #6c757d;
    width: 100%;
    word-break: break-all;
    height: 260px;
    cursor: pointer;
    background: url([[pix:tiny_h5p|icon-white]]) center center / 100px auto no-repeat #6c757d;
}
