<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'tiny_media', language 'en'.
 *
 * @package    tiny_media
 * @copyright  2022 Andrew Lyons <andrew@nicols.co.uk>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addcaptionstrack'] = 'Add caption track';
$string['addchapterstrack'] = 'Add chapter track';
$string['adddescriptionstrack'] = 'Add description track';
$string['addfilesdrop'] = 'Drag and drop an image to upload, or click to select';
$string['addmediafilesdrop'] = 'Drag and drop audio/video file to upload, or click to select';
$string['addmediathumbnaildrop'] = 'Drag and drop image thumbnail file to upload, or click to select';
$string['addmetadatatrack'] = 'Add metadata track';
$string['addsubtitlestrack'] = 'Add subtitle track';
$string['addurl'] = 'Add';
$string['audio'] = 'Audio';
$string['autoplay'] = 'Autoplay';
$string['back'] = 'Back';
$string['browseembedimagerepositories'] = 'Browse repositories';
$string['browserepositories'] = 'Browse repositories...';
$string['captions'] = 'Captions';
$string['captions_help'] = 'Captions may be used to describe everything happening in the track, including non-verbal sounds such as a phone ringing.';
$string['captionssourcelabel'] = 'Caption track URL';
$string['changethumbnail'] = 'Change thumbnail';
$string['chapters'] = 'Chapters';
$string['chapters_help'] = 'Chapter titles may be provided for use in navigating the media resource.';
$string['chapterssourcelabel'] = 'Chapter track URL';
$string['constrain'] = 'Keep proportion';
$string['controls'] = 'Show controls';
$string['customsize_help'] = 'For best viewing experience, the video\'s width and height will adjust together, keeping the original aspect ratio.';
$string['default'] = 'Default';
$string['deleteembeddedthumbnail'] = 'Delete thumbnail';
$string['deleteembeddedthumbnailwarning'] = 'Are you sure you want to remove the embedded thumbnail?';
$string['deleteimage'] = 'Delete image';
$string['deleteimagewarning'] = 'Are you sure you want to remove the image?';
$string['deletemedia'] = 'Delete media';
$string['deletemediathumbnail'] = 'Delete media thumbnail';
$string['deletemediathumbnailwarning'] = 'Are you sure you want to remove the media thumbnail?';
$string['deletemediawarning'] = 'Are you sure you want to remove the media?';
$string['deleteselected'] = 'Delete selected files';
$string['deletethumbnail'] = 'Delete custom thumbnail';
$string['descriptions'] = 'Descriptions';
$string['descriptions_help'] = 'Audio descriptions may be used to provide a narration which explains visual details not apparent from the audio alone.';
$string['descriptionssourcelabel'] = 'Description track URL';
$string['enteralt'] = 'How would you describe this image to someone who can\'t see it?';
$string['entertitle'] = 'Media title';
$string['enterurl'] = 'Add via URL';
$string['enterurlor'] = 'Or add via URL';
$string['filemanager'] = 'File manager';
$string['hasmissingfiles'] = 'Warning! The following files that are referenced in the text area appear to be missing:';
$string['height'] = 'Height';
$string['imagebuttontitle'] = 'Image';
$string['imagedetails'] = 'Image details';
$string['imageproperties'] = 'Image properties';
$string['imageurlrequired'] = 'An image must have a valid URL.';
$string['insertimage'] = 'Insert image';
$string['insertmedia'] = 'Insert media';
$string['insertmediathumbnail'] = 'Insert media thumbnail';
$string['label'] = 'Label';
$string['languagesavailable'] = 'Languages available';
$string['languagesinstalled'] = 'Languages installed';
$string['linkcustomsize_help'] = 'For best viewing experience, the link media\'s width and height will adjust together with 16:9 aspect ratio.';
$string['loading'] = 'Preparing the image';
$string['loadingembedthumbnail'] = 'Preparing the embed thumbnail';
$string['loadingmedia'] = 'Preparing the media';
$string['loop'] = 'Loop';
$string['managefiles'] = 'Manage files';
$string['media:use'] = 'Use TinyMCE insert media';
$string['mediabuttontitle'] = 'Multimedia';
$string['mediadetails'] = 'Media details';
$string['medialinktypeselector'] = 'Select media type';
$string['mediamanagerbuttontitle'] = 'Media manager';
$string['mediamanagerproperties'] = 'Media manager';
$string['medianotavailable'] = 'Media is not available';
$string['medianotavailabledesc'] = 'Error loading the following media url: {$a}';
$string['mediaurlrequired'] = 'An audio/video must have a valid URL.';
$string['metadata'] = 'Metadata';
$string['metadata_help'] = 'Metadata tracks, for use from a script, may be used only if the player supports metadata.';
$string['metadatasourcelabel'] = 'Metadata track URL';
$string['missingfiles'] = 'Missing files';
$string['subtitlesandcaptions'] = 'Subtitles and captions';
$string['mute'] = 'Muted';
$string['next'] = 'Next';
$string['nofilepicker'] = 'Paste a link to an audio/video file in the field below or<br>click the Browse Repositories button.';
$string['nofilepickerrepository'] = 'Paste an {$a} link in the field below.';
$string['onlymediafiles'] = 'Accepts only media files';
$string['pluginname'] = 'Insert media';
$string['presentation'] = 'This image is decorative only';
$string['presentationoraltrequired'] = 'An image must have a description, unless it is marked as decorative only.';
$string['privacy:metadata'] = 'The media plugin for TinyMCE does not store any personal data.';
$string['remove'] = 'Remove';
$string['repositorynotpermitted'] = 'Paste an image link in the field below.';
$string['repositoryuploadnotpermitted'] = 'Paste an image link in the field below or<br>click the Browse Repositories button.';
$string['saveimage'] = 'Save';
$string['size'] = 'Width x height (in pixels)';
$string['sizecustom'] = 'Custom size';
$string['sizecustom_help'] = 'This image is just a preview. Changes to its size will be visible after you save it.';
$string['sizeoriginal'] = 'Original size';
$string['srclang'] = 'Language';
$string['subtitles'] = 'Subtitles';
$string['subtitles_help'] = 'Subtitles may be used to provide a transcription or translation of the dialogue.';
$string['subtitlessourcelabel'] = 'Subtitle track URL';
$string['thumbnail'] = 'Media thumbnail';
$string['tracks'] = 'Subtitles and captions';
$string['unusedfilesdesc'] = 'The following embedded files are not used in the text area:';
$string['unusedfilesheader'] = 'Unused files';
$string['unusedfilesremovalnotice'] = 'Any unused files will be automatically deleted when saving changes.';
$string['uploading'] = 'Uploading';
$string['uploadthumbnail'] = 'Add custom thumbnail';
$string['video'] = 'Video';
$string['width'] = 'Width';

// Deprecated since Moodle 4.5.
$string['addsource'] = 'Add alternative source';
$string['addsource_help'] = 'You are recommended to provide an alternative media source, as desktop and mobile browsers support different file formats.';
$string['advancedsettings'] = 'Advanced settings';
$string['audiosourcelabel'] = 'Audio source URL';
$string['browserepositoriesimage'] = 'Browse repositories';
$string['createmedia'] = 'Insert media';
$string['displayoptions'] = 'Display options';
$string['entername'] = 'Name';
$string['entersource'] = 'Source URL';
$string['helplinktext'] = 'Media helper';
$string['link'] = 'Link';
$string['tracks_help'] = 'Subtitles, captions, chapters and descriptions can be added via a WebVTT (Web Video Text Tracks) format file. Track labels will be shown in the selection drop-down menu. For each type of track, any track set as default will be pre-selected at the start of the video.';
$string['updatemedia'] = 'Update media';
$string['videoheight'] = 'Video height';
$string['videosourcelabel'] = 'Video source URL';
$string['videowidth'] = 'Video width';
