{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_recordrtc/insert_recording

    Insert recording template.

    Example context (json):
    {

    }
}}
<div class="tiny_recordrtc container-fluid">
    <div  data-purpose="security-warning" class="row hide">
        <div class="col-12">
            <div id="alert-danger" class="alert alert-danger">
                <strong>
                    {{#str}} insecurealert_title, tiny_recordrtc {{/str}}
                </strong>
                {{#str}} insecurealert, tiny_recordrtc {{/str}}
            </div>
        </div>
    </div>
    <div data-purpose="player-container" class="row hide">
        {{#isaudio}}
            <div class="col-1"></div>
            <div class="col-10">
                <audio id="player"></audio>
            </div>
            <div class="col-1"></div>
        {{/isaudio}}
        {{^isaudio}}
            <div class="col-12">
                <video id="player"></video>
            </div>
        {{/isaudio}}
    </div>
    <div data-purpose="start-stop-container" class="row">
        <div class="col-1"></div>
        <div class="col-10">
            <button id="start-stop" class="btn btn-lg btn-outline-danger w-100" data-action-"startstop">
                {{#str}} startrecording, tiny_recordrtc {{/str}}
            </button>
        </div>
        <div class="col-1"></div>
    </div>
    <div data-purpose="attach-container" class="row hide">
        <div class="col-3"></div>
        <div class="col-6">
            <button id="upload" class="btn btn-primary w-100" data-action-"attach">
                {{#str}} attachrecording, tiny_recordrtc {{/str}}
            </button>
        </div>
        <div class="col-3"></div>
    </div>
</div>
