{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/insert_image_modal

    Insert image template.

    Example context (json):
    {
        "uniqid": 0,
        "elementid": "exampleId",
        "loading": {},
        "title": "Insert image"
    }
}}
{{< core/modal }}
    {{$body}}
        <form class="tiny_image_form">
            <!-- URL warning -->
            <div role="alert" class="d-none alert alert-warning mb-1 tiny_image_urlwarning">
                <label>
                    {{#str}} imageurlrequired, tiny_media {{/str}}
                </label>
            </div>
            <!-- Presentation warning -->
            <div role="alert" class="d-none tiny_image_altwarning alert alert-warning mb-1">
                <label>
                    {{#str}} presentationoraltrequired, tiny_media {{/str}}
                </label>
            </div>
            <!-- Preloader icon -->
            <div role="progressbar" class="d-none tiny_image_loader lead border rounded text-center">
                <div class="tiny_image_loader_container d-flex flex-column justify-content-center">
                    <span data-region="loading-icon-container">
                        {{> core/loading }}
                    </span>
                    <div>{{#str}} loading, tiny_media {{/str}}</div>
                </div>
            </div>
            <div class="tiny_image_body_template"></div>
        </form>
    {{/body}}

    {{$footer}}
        <div class="tiny_image_footer_template container"></div>
    {{/footer}}
{{/ core/modal }}
