{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_accessibilitychecker/warning_content

    Display a content warning message

    Example context (json):
    {
        "warnings": [
            {
                "description": "This is a warning message",
                "nodeData": [{
                    "text": "Text of content",
                    "src": "https://example.com/image.png",
                    "nodeName": "IMG",
                    "nodeIndex": 0
                }]
            }
        ]
    }
}}
<div class="warning-desc">
    {{#warnings}}
            <p>{{{description}}}</p>
            <ol class="accessibilitywarnings">
                {{#nodeData}}
                    <li>
                        {{#text}}
                            {{text}}
                        {{/text}}
                        {{#src}}
                            <img alt="" src="{{src}}" width="100"/>
                        {{/src}}
                        - <a href="#" data-action="highlightfault" data-node-name="{{nodeName}}" data-node-index="{{nodeIndex}}">{{#str}} viewissue, tiny_accessibilitychecker {{/str}}</a>
                    </li>
                {{/nodeData}}
            </ol>
    {{/warnings}}
    {{^warnings}}
        <p>{{#str}} nowarnings, tiny_accessibilitychecker {{/str}}</p>
    {{/warnings}}
</div>
