{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tiny_media/insert_image_modal_insert_footer

    Insert image footer template.

    Example context (json):
    {
        "showdropzone": true,
        "elementid": "exampleId",
        "src": "https://moodle.org/logo.png",
        "showfilepicker": true
    }

}}
<div class="row">
    <!-- First Column -->
    <div class="col-md-6 d-flex align-items-center p-0">
        <!-- Row 1: URL related label -->
        {{#showdropzone}}
        <label for="{{elementid}}_tiny_image_urlentry" class="tiny_image_url_label me-1">{{#str}} enterurlor, tiny_media {{/str}}</label>
        {{/showdropzone}}
        {{^showdropzone}}
        <label for="{{elementid}}_tiny_image_urlentry" class="tiny_image_url_label me-1">{{#str}} enterurl, tiny_media {{/str}}</label>
        {{/showdropzone}}
        <!-- Row 2: URL entry input. Needed by the insert image step and image details step if the image URL source from external -->
        <input name="urlentry" class="tiny_image_urlentry form-control w-50 me-1" type="url" id="{{elementid}}_tiny_image_urlentry" size="32" value="{{src}}">
        <!-- Row 3: Add button. Needed by the insert image step -->
        <button disabled class="tiny_image_addurl btn btn-secondary me-1" type="button">{{#str}} addurl, tiny_media {{/str}}</button>
    </div>
    <!-- Column 2: Saving, canceling, browsing repositories buttons -->
    <div class="col-md-6 text-end mt-2 mt-md-0 p-0">
        <!-- Row 1: Cancel button -->
        <button type="button" class="btn btn-secondary" data-action="cancel">{{#str}} cancel, moodle {{/str}}</button>
        {{#showfilepicker}}
            <!-- Row 3: Browse repositories button -->
            <button class="openimagebrowser btn btn-secondary" type="button">{{#str}} browseembedimagerepositories, tiny_media {{/str}}</button>
        {{/showfilepicker}}
    </div>
</div>

